\name{packCircles}
\alias{packCircles}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Function to create packed circles diagrams
}
\description{
packCircles uses the circleProgressiveLayout function from package packcircles for creating packed circles diagrams. 
}
\usage{
packCircles(x, sizecol = NULL, sizetype = c("area", "radius"), 
     groupcol = NULL, labcol = NULL, 
     groupsort = FALSE, decreasing = FALSE, 
     pal = blues9[3:9], col.line = NULL, lwd = 2, 
     col.lab = NULL, 
     col.lab.defaults = c(par("bg"), par("fg")),
     cex.lab = par("cex.lab"), 
     main=NULL, sub=NULL, plot = TRUE, annot = TRUE, add = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
a vector, matrix or data frame, see Details section for different possibilities.
}
  \item{sizecol}{
for matrix or data frame \code{x}: column name or column number for circle sizes; non-NULL values are ignored for vector-valued categorical \code{x} and throw an error for vector-valued numerical \code{x}}
  \item{sizetype}{
to be given to function \code{link{circleProgressiveLayout}}; default: \code{"area"}; alternative: \code{"radius"}
}
  \item{groupcol}{
for matrix or data frame \code{x}: \code{NULL} or column name or column number for group column;\cr
for vector \code{x}: \code{NULL} or same length vector of group memberships 
}
  \item{labcol}{
for matrix or data frame \code{x}: \code{NULL} or column name or column number for labels column;\cr
for vector \code{x}: \code{NULL} or same length vector of circle labels 
}
  \item{groupsort}{
logical, whether or not the rows of x are sorted before circle packing; sorting makes a group stick together in terms of centrality in the diagram (first group mostly in the center, last group mostly on the outside); CAUTION: if \code{groupsort=TRUE}, great care is needed with \emph{subsequent} labeling of circles!
}
  \item{decreasing}{
logical: should the sort order for \code{groupsort} be decreasing? If \code{TRUE}, a decreasing sort order is chosen, and the values of all color vectors are resorted by function \code{\link{rev}}.
}
  \item{pal}{
vector of colors to be used for circle coloring; colors are determined according to the value of \code{groupcol}, which should be a categorical variable (if not a factor, it is forced into one by \code{\link{as.factor}}); if there are too few colors, \code{pal} is 
}
  \item{col.line}{
color of the circle border; default: equal to fill color; single color or vector of the same length as \code{pal}
}
  \item{lwd}{
line width of the circle border
}
  \item{col.lab}{
label color; default \code{NULL} translates to the more suitable of the two colors in \code{col.lab.default}; single color or vector of same length as \code{pal}; if too few colors are specified, these are recycled with a warning
}
  \item{col.lab.defaults}{
default label colors (two colors needed); default: \code{c(par("bg"),par("fg"))}; if \code{col.lab} is not specified, one of these is used for each label, depending which color gives more contrast (in terms of distance in RGB space) versus the fill color
}
  \item{cex.lab}{
text label size, single number or vector; a single number specifies the maximum text label size (default: \code{par("cex.lab")}); if a single number is given, wide labels are downsized to fit into their respective circles; this may not work perfectly for multiline labels
}
  \item{main}{
title; if \code{NULL}, a default title is generated; suppressed, if \code{annot=FALSE}
}
  \item{sub}{
sub text; if \code{NULL}, a default text is generated; suppressed, if \code{annot=FALSE}
}
  \item{plot}{
logical, whether or not to create the plot
}
  \item{annot}{
logical, whether or not to include \code{main} and \code{sub}; if \code{FALSE}, the margins are also used for plotting (set to \code{mar=c(0,0,0,0)} during function execution, reverted on exit)
}
  \item{add}{
logical, whether or not to add to an existing figure
}
  \item{\dots}{
further parameters to be given to functions \code{plot}, \code{symbols} and \code{text}, e.g. \code{main} or \code{sub}
}
}
\details{
If \code{x} is a matrix or data frame, \code{sizecol}, \code{groupcol} and \code{labcol}, if given, reference a column of \code{x} by name or index number. 
If \code{x} is a single character vector or factor, its frequency table is visualized by circle sizes. \code{sizecol}, \code{groupcol} and \code{labcol} specifications are ignored in this case.

If \code{x} is a single numeric vector, it takes the role of \code{sizecol}, and specifications of \code{groupcol} and \code{labcol} of the same length as \code{x} are accepted. 

If \code{x} is a named vector or the matrix or data frame has row names, these are used for \code{labcol}, if \code{labcol=NULL}; if this is undesired, these can e.g. be suppressed by choosing \code{col.lab} equally to \code{pal}, with small enough \code{cex.lab}.

The circle packing algorithm implemented in function \code{\link{circleProgressiveLayout}} from package \pkg{packcircles} is used (that package quotes Wang, Wang, Dai and Wang (2006) for the algorithm).

Packed circles diagrams (like pie charts) are not necessarily the best visualization for a rational overview of the facts. They can, however, be made to look attractive, and they invite detailed looks at aspects of the data. 

In my view, the best use is to omit circle labelling and have mouseovers instead. This can e.g. be done with package \pkg{sendplot} (yields a standalone html file).
}
\value{
The function returns a data frame with the columns \code{x}, \code{y}, \code{radius}, and all the columns needed for producing the chart; if \code{groupsort=TRUE}, there is also a column named \code{orig.order} that gives the position of each row in the ingoing data.
}
\references{
Weixin Wang, Hui Wang, Guozhong Dai, and Hongan Wang (2006). Visualization of large hierarchical data by circle packing. \emph{Proceedings of the SIGCHI Conference on Human Factors in Computing Systems}, pp. 517-520. \url{https://dl.acm.org/citation.cfm?id=1124851}
}
\author{
Urike Groemping, Beuth University of Applied Sciences Berlin
}

\seealso{
See also \code{\link{packcircles}} for the algorithm behind this function, \pkg{sendplot} for creating mouseovers, and \pkg{D3partitionR} for interactive hierarchical circular treemaps.
}
\examples{
## visualizing a frequency table for a factor variable
## refugee countries of origin 2014 (first applications for asylum)
## source: "Das Bundesamt in Zahlen 2016. Asyl, Migration und Integration"
## (official report of the German Office for Migration and Refugees)
country <- c(rep("Syria", 39332), rep("Serbia", 17172), rep("Eritrea", 13198),
    rep("Afghanistan", 9115), rep("Albania", 7865), rep("Kosovo", 6908), 
    rep("BosniaHerzeg.", 5705), rep("Macedonia", 5614), 
    rep("Somalia", 5528), rep("Iraq", 5345))
(nFirst10 <- length(country)) ## for info 10 most frequent countries
(nOther <- 173072 - nFirst10)  ## for info other countries
country <- c(country, rep("Other", nOther))

## directly using the categorical variable 
omar <- par(mar=c(2,2,4,2))
packCircles(country, main="Refugee countries of origin in 2014", cex=0.8, 
    sub="", col.lab=blues9[9], col.main=blues9[9])
par(omar)

## using a table allows differential fill colors
tab <- table(country)  ## is seen as a numeric vector
pct <- 100*tab/sum(tab)
labs <- paste(names(tab), paste0("(",round(pct,1),"\%)"), sep="\n")
omar <- par(mar=c(2,2,4,2))
erg <- packCircles(tab, groupcol=tab, labcol=labs, 
      main="Refugee countries of origin in 2014", cex=0.8,
      sub="", pal=grey.colors(11), col.line="grey20", col.main="grey20")
par(omar)

## sorted by group variable
omar <- par(mar=c(2,2,4,2))
erg <- packCircles(tab, groupcol=tab, groupsort=TRUE, labcol=labs, 
      main="Refugee countries of origin in 2014", cex=0.8,
      sub="", pal=grey.colors(11),
      col.line=blues9[9], col.main=blues9[9], 
      col.lab.defaults=c(blues9[1],blues9[9]), font=2)
par(omar)

## reversed sort order makes better use of space (overall larger circles)
omar <- par(mar=c(2,2,4,2))
erg <- packCircles(tab, groupcol=tab, groupsort=TRUE, decreasing=TRUE, 
      labcol=labs, 
      main="Refugee countries of origin in 2014", cex=0.8,
      sub="", pal=grey.colors(11),
      col.line=blues9[9], col.main=blues9[9], 
      col.lab.defaults=c(blues9[1],blues9[9]), font=2)
par(omar)

## reverse sort order, but keep original coloring
omar <- par(mar=c(2,2,4,2))
erg <- packCircles(tab, groupcol=tab, groupsort=TRUE, decreasing=TRUE, 
      labcol=labs, 
      main="Refugee countries of origin in 2014", cex=0.8,
      sub="", pal=rev(grey.colors(11)),
      col.line=blues9[9], col.main=blues9[9], 
      col.lab.defaults=c(blues9[1],blues9[9]), font=2)
par(omar)

df <- as.data.frame(tab)  ## has columns country and Freq
df <- cbind(df, label=labs)
omar <- par(mar=c(2,2,4,2))
erg <- packCircles(df, sizecol="Freq", groupcol="Freq", groupsort=TRUE, decreasing=TRUE, 
    labcol="label", main="Refugee countries of origin in 2014", cex=0.8,
      sub="", pal=grey.colors(11), col.line=blues9[9], col.main=blues9[9], 
      col.lab.defaults=c(blues9[1],blues9[9]))
par(omar)

## certainly more informative
omar <- par(mar=c(3,8,4,2))
bp <- barplot(sort(tab), horiz=TRUE, 
    main="Refugee countries of origin in 2014", las=1, xlim=c(0,60000))
text(sort(tab), bp, adj=c(-0.15, 0.5), 
     label=paste0(round(100*sort(tab)/sum(tab),1), "\%"), cex=0.8, xpd=NA)
par(omar)

## example with many individual objects 
## from a few catogories
\dontrun{
require(DAAG)  ## rainforest data
require(pals)  ## for color palette
cs <- brewer.dark2(4)
packCircles(rainforest, sizecol="dbh", groupcol="species", groupsort=TRUE,
   pal=cs, col.lab=cs, main="dbh for rainforest trees of four species",
   sub=paste(c("min","max"), range(rainforest$dbh), sep="=", collapse=", "))
legend("bottom", fill=cs, border=cs, legend=levels(rainforest$species), 
   horiz=TRUE, cex=0.8, xpd=NA, inset=-0.1)

## species are almost sorted in the original data frame
## what if randomly sorted?
set.seed(3273)
packCircles(rainforest[sample(1:nrow(rainforest)),], 
   sizecol="dbh", groupcol="species", 
   pal=cs, col.lab=cs, main="dbh for rainforest trees of four species",
   sub=paste("circle area=dbh value", 
   paste(c("min","max"), range(rainforest$dbh), sep="=", collapse=", "), 
   sep="; "))
legend("bottomleft", fill=cs, border=cs, legend=levels(rainforest$species), 
   cex=0.8, xpd=NA, inset=-0.1, box.col=par("bg"))

## of course more scientific
   ## but coloring boxes would give misleading emphasis less frequent 
   ## fourth specie
boxplot(dbh~species, rainforest, ylab="dbh", main="dbh for rainforest trees of four species")
mtext(side=1, at=1:4, line=2, paste("n =", table(rainforest$species)))

## here, species can be colored
stripchart(dbh~species, rainforest, method="stack", ylab="dbh", 
   main="dbh for rainforest trees of four species", col=cs, pch=16, 
   vertical=TRUE)
mtext(side=1, at=1:4, line=2, paste("n =", table(rainforest$species)))
}

## visualizing a quantitative variable, 
## ordered and colored by another quantitative variable
## grey.colors gives suitable colors for this purpose 
## (however, the colors are by order only, they don't reflect step sizes, 
##    unless the user takes special care to provide a palette that does that)

## default annotation
packCircles(swiss, sizecol="Fertility", pal=grey.colors(47), 
    groupcol="Catholic", groupsort=TRUE, cex.lab=0.8)

## no annotation
packCircles(swiss, sizecol="Fertility", pal=grey.colors(47), 
    groupcol="Catholic", groupsort=TRUE, cex.lab=0.1, col.lab=grey.colors(47))

## more detailed labels, added legend circle
labels <- paste(round(swiss$Fertility), rownames(swiss), round(swiss$Catholic), sep="\n")
swiss2 <- cbind(swiss, lab = labels)
packCircles(swiss2, sizecol="Fertility", pal=grey.colors(47), 
    groupcol="Catholic", groupsort=TRUE, labcol="lab", cex=0.7, 
    main="Swiss provinces: Fertility (size) and Catholic percentage (color)",
    sub="")
symbols(-35, -45, circles=10, inches=FALSE, bg="grey70", fg="grey80", 
    add=TRUE, xpd=NA)
text(-35, -45, 
    paste("Fertility (Ig)","County name", "Catholic \%", sep="\n"), 
    cex=0.7, xpd=NA)

## certainly generally more informative
plot(Fertility~Catholic, swiss, type="n")
text(swiss$Catholic, swiss$Fertility, rownames(swiss), cex=0.7)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ hplot }% use one of  RShowDoc("KEYWORDS")
\keyword{ aplot }% __ONLY ONE__ keyword per line
